@@ -1,5 +1,7 @@
+from __future__ import annotations
+
+import enum
 import os
-import uuid
 from itertools import chain
 
 import pytest
@@ -104,125 +106,135 @@ def test_group_from_list(runner):
     assert result.output == "sub"
 
 
-def test_basic_option(runner):
+@pytest.mark.parametrize(
+    ("args", "expect"),
+    [
+        ([], "S:[no value]"),
+        (["--s=42"], "S:[42]"),
+        (["--s"], "Error: Option '--s' requires an argument."),
+        (["--s="], "S:[]"),
+        (["--s=\N{SNOWMAN}"], "S:[\N{SNOWMAN}]"),
+    ],
+)
+def test_string_option(runner, args, expect):
     @click.command()
-    @click.option("--foo", default="no value")
-    def cli(foo):
-        click.echo(f"FOO:[{foo}]")
+    @click.option("--s", default="no value")
+    def cli(s):
+        click.echo(f"S:[{s}]")
 
-    result = runner.invoke(cli, [])
-    assert not result.exception
-    assert "FOO:[no value]" in result.output
+    result = runner.invoke(cli, args)
+    assert expect in result.output
 
-    result = runner.invoke(cli, ["--foo=42"])
-    assert not result.exception
-    assert "FOO:[42]" in result.output
-
-    result = runner.invoke(cli, ["--foo"])
-    assert result.exception
-    assert "Option '--foo' requires an argument." in result.output
-
-    result = runner.invoke(cli, ["--foo="])
-    assert not result.exception
-    assert "FOO:[]" in result.output
-
-    result = runner.invoke(cli, ["--foo=\N{SNOWMAN}"])
-    assert not result.exception
-    assert "FOO:[\N{SNOWMAN}]" in result.output
+    if expect.startswith("Error:"):
+        assert result.exception is not None
+    else:
+        assert result.exception is None
 
 
-def test_int_option(runner):
+@pytest.mark.parametrize(
+    ("args", "expect"),
+    [
+        ([], "I:[84]"),
+        (["--i=23"], "I:[46]"),
+        (["--i=x"], "Error: Invalid value for '--i': 'x' is not a valid integer."),
+    ],
+)
+def test_int_option(runner, args, expect):
     @click.command()
-    @click.option("--foo", default=42)
-    def cli(foo):
-        click.echo(f"FOO:[{foo * 2}]")
+    @click.option("--i", default=42)
+    def cli(i):
+        click.echo(f"I:[{i * 2}]")
 
-    result = runner.invoke(cli, [])
-    assert not result.exception
-    assert "FOO:[84]" in result.output
+    result = runner.invoke(cli, args)
+    assert expect in result.output
 
-    result = runner.invoke(cli, ["--foo=23"])
-    assert not result.exception
-    assert "FOO:[46]" in result.output
-
-    result = runner.invoke(cli, ["--foo=bar"])
-    assert result.exception
-    assert "Invalid value for '--foo': 'bar' is not a valid integer." in result.output
+    if expect.startswith("Error:"):
+        assert result.exception is not None
+    else:
+        assert result.exception is None
 
 
-def test_uuid_option(runner):
+@pytest.mark.parametrize(
+    ("args", "expect"),
+    [
+        ([], "U:[ba122011-349f-423b-873b-9d6a79c688ab]"),
+        (
+            ["--u=821592c1-c50e-4971-9cd6-e89dc6832f86"],
+            "U:[821592c1-c50e-4971-9cd6-e89dc6832f86]",
+        ),
+        (["--u=x"], "Error: Invalid value for '--u': 'x' is not a valid UUID."),
+    ],
+)
+def test_uuid_option(runner, args, expect):
     @click.command()
     @click.option(
         "--u", default="ba122011-349f-423b-873b-9d6a79c688ab", type=click.UUID
     )
     def cli(u):
-        assert type(u) is uuid.UUID
         click.echo(f"U:[{u}]")
 
-    result = runner.invoke(cli, [])
-    assert not result.exception
-    assert "U:[ba122011-349f-423b-873b-9d6a79c688ab]" in result.output
+    result = runner.invoke(cli, args)
+    assert expect in result.output
 
-    result = runner.invoke(cli, ["--u=821592c1-c50e-4971-9cd6-e89dc6832f86"])
-    assert not result.exception
-    assert "U:[821592c1-c50e-4971-9cd6-e89dc6832f86]" in result.output
-
-    result = runner.invoke(cli, ["--u=bar"])
-    assert result.exception
-    assert "Invalid value for '--u': 'bar' is not a valid UUID." in result.output
+    if expect.startswith("Error:"):
+        assert result.exception is not None
+    else:
+        assert result.exception is None
 
 
-def test_float_option(runner):
+@pytest.mark.parametrize(
+    ("args", "expect"),
+    [
+        ([], "F:[42.0]"),
+        ("--f=23.5", "F:[23.5]"),
+        ("--f=x", "Error: Invalid value for '--f': 'x' is not a valid float."),
+    ],
+)
+def test_float_option(runner, args, expect):
     @click.command()
-    @click.option("--foo", default=42, type=click.FLOAT)
-    def cli(foo):
-        assert type(foo) is float
-        click.echo(f"FOO:[{foo}]")
+    @click.option("--f", default=42.0)
+    def cli(f):
+        click.echo(f"F:[{f}]")
 
-    result = runner.invoke(cli, [])
-    assert not result.exception
-    assert "FOO:[42.0]" in result.output
+    result = runner.invoke(cli, args)
+    assert expect in result.output
 
-    result = runner.invoke(cli, ["--foo=23.5"])
-    assert not result.exception
-    assert "FOO:[23.5]" in result.output
-
-    result = runner.invoke(cli, ["--foo=bar"])
-    assert result.exception
-    assert "Invalid value for '--foo': 'bar' is not a valid float." in result.output
+    if expect.startswith("Error:"):
+        assert result.exception is not None
+    else:
+        assert result.exception is None
 
 
-def test_boolean_option(runner):
-    for default in True, False:
+@pytest.mark.parametrize("default", [True, False])
+@pytest.mark.parametrize(
+    ("args", "expect"), [(["--on"], True), (["--off"], False), ([], None)]
+)
+def test_boolean_switch(runner, default, args, expect):
+    @click.command()
+    @click.option("--on/--off", default=default)
+    def cli(on):
+        return on
 
-        @click.command()
-        @click.option("--with-foo/--without-foo", default=default)
-        def cli(with_foo):
-            click.echo(with_foo)
+    if expect is None:
+        expect = default
 
-        result = runner.invoke(cli, ["--with-foo"])
-        assert not result.exception
-        assert result.output == "True\n"
-        result = runner.invoke(cli, ["--without-foo"])
-        assert not result.exception
-        assert result.output == "False\n"
-        result = runner.invoke(cli, [])
-        assert not result.exception
-        assert result.output == f"{default}\n"
+    result = runner.invoke(cli, args, standalone_mode=False)
+    assert result.return_value is expect
 
-    for default in True, False:
 
-        @click.command()
-        @click.option("--flag", is_flag=True, default=default)
-        def cli(flag):
-            click.echo(flag)
+@pytest.mark.parametrize("default", [True, False])
+@pytest.mark.parametrize(("args", "expect"), [(["--f"], True), ([], False)])
+def test_boolean_flag(runner, default, args, expect):
+    @click.command()
+    @click.option("--f", is_flag=True, default=default)
+    def cli(f):
+        return f
 
-        result = runner.invoke(cli, ["--flag"])
-        assert not result.exception
-        assert result.output == f"{not default}\n"
-        result = runner.invoke(cli, [])
-        assert not result.exception
-        assert result.output == f"{default}\n"
+    if default:
+        expect = not expect
+
+    result = runner.invoke(cli, args, standalone_mode=False)
+    assert result.return_value is expect
 
 
 @pytest.mark.parametrize(
@@ -394,6 +406,82 @@ def test_choice_argument(runner):
     assert "{foo|bar|baz}" in result.output
 
 
+def test_choice_argument_enum(runner):
+    class MyEnum(str, enum.Enum):
+        FOO = "foo-value"
+        BAR = "bar-value"
+        BAZ = "baz-value"
+
+    @click.command()
+    @click.argument("method", type=click.Choice(MyEnum, case_sensitive=False))
+    def cli(method: MyEnum):
+        assert isinstance(method, MyEnum)
+        click.echo(method)
+
+    result = runner.invoke(cli, ["foo"])
+    assert result.output == "foo-value\n"
+    assert not result.exception
+
+    result = runner.invoke(cli, ["meh"])
+    assert result.exit_code == 2
+    assert (
+        "Invalid value for '{foo|bar|baz}': 'meh' is not one of 'foo',"
+        " 'bar', 'baz'." in result.output
+    )
+
+    result = runner.invoke(cli, ["--help"])
+    assert "{foo|bar|baz}" in result.output
+
+
+def test_choice_argument_custom_type(runner):
+    class MyClass:
+        def __init__(self, value: str) -> None:
+            self.value = value
+
+        def __str__(self) -> str:
+            return self.value
+
+    @click.command()
+    @click.argument(
+        "method", type=click.Choice([MyClass("foo"), MyClass("bar"), MyClass("baz")])
+    )
+    def cli(method: MyClass):
+        assert isinstance(method, MyClass)
+        click.echo(method)
+
+    result = runner.invoke(cli, ["foo"])
+    assert not result.exception
+    assert result.output == "foo\n"
+
+    result = runner.invoke(cli, ["meh"])
+    assert result.exit_code == 2
+    assert (
+        "Invalid value for '{foo|bar|baz}': 'meh' is not one of 'foo',"
+        " 'bar', 'baz'." in result.output
+    )
+
+    result = runner.invoke(cli, ["--help"])
+    assert "{foo|bar|baz}" in result.output
+
+
+def test_choice_argument_none(runner):
+    @click.command()
+    @click.argument(
+        "method", type=click.Choice(["not-none", None], case_sensitive=False)
+    )
+    def cli(method: str | None):
+        assert isinstance(method, str) or method is None
+        click.echo(method)
+
+    result = runner.invoke(cli, ["not-none"])
+    assert not result.exception
+    assert result.output == "not-none\n"
+
+    # None is not yet supported.
+    result = runner.invoke(cli, ["none"])
+    assert result.exception
+
+
 def test_datetime_option_default(runner):
     @click.command()
     @click.option("--start_date", type=click.DateTime())
@@ -547,3 +635,20 @@ def test_summary_line(runner):
     result = runner.invoke(cli, ["--help"])
     assert "Summary line without period" in result.output
     assert "Here is a sentence." not in result.output
+
+
+def test_help_invalid_default(runner):
+    cli = click.Command(
+        "cli",
+        params=[
+            click.Option(
+                ["-a"],
+                type=click.Path(exists=True),
+                default="not found",
+                show_default=True,
+            ),
+        ],
+    )
+    result = runner.invoke(cli, ["--help"])
+    assert result.exit_code == 0
+    assert "default: not found" in result.output
